#!/usr/bin/perl

use Test::More qw(no_plan);

use FindBin qw($Bin);
use lib "$Bin/lib", "$Bin/../lib";
use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";
use strict;
use Opsview::Test;
use Opsview::TestCGI;

Opsview::Test->debug(1) if @ARGV;

my $sth;
my $dbh = Opsview->db_Main;
ok( defined $dbh, "Connect to db" );

my $rc;

# Tests on hosttemplates
my $rows = $dbh->selectrow_array("SELECT COUNT(*) FROM hosttemplates");
$rc = post("type=hosttemplate action=new name=another$$ description=this$$");
ok( $rc == 0, "Added host template" );
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM hosttemplates") == ( $rows + 1 ), "Row added to db" );

# This test is invalid as it is considered as a single action now
#$rc = post("type=hosttemplate action=new name=another$$ description=this$$");
#ok( $rc != 0, "Added same host template, expected failure" );
#ok( $dbh->selectrow_array("SELECT COUNT(*) FROM hosttemplates") == ( $rows + 1 ), "No extra rows added to db" );

$rc = post('type=hosttemplate action=new');
ok( $rc != 0, "Missing name" );
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM hosttemplates") == ( $rows + 1 ), "No extra rows added to db" );

my $scid1 = $dbh->selectrow_array("SELECT MIN(id) FROM servicechecks");
my $scid2 = $dbh->selectrow_array("SELECT MAX(id) FROM servicechecks");

$rc = post("type=hosttemplate action=new name=tricky$$ servicechecks=$scid1 servicechecks=$scid2");
ok( $rc == 0, "Adding servicechecks $scid1 & $scid2" );
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM hosttemplates") == ( $rows + 2 ), "Row added to db" );
my $htid = $dbh->selectrow_array("SELECT id FROM hosttemplates WHERE name='tricky$$'");
my $cols = $dbh->selectcol_arrayref("SELECT servicecheckid FROM hosttemplateservicechecks WHERE hosttemplateid=$htid ORDER BY servicecheckid");
$_ = join( ",", @$cols );
ok( $_ eq "$scid1,$scid2", "double servicechecks added to hosttemplates correctly in DB" );

#
# Exceptions testing
#
cmp_ok( $dbh->selectrow_array("SELECT COUNT(*) FROM servicecheckhosttemplateexceptions WHERE hosttemplate = $htid"), "==", 0, "No exceptions for hosttemplate added" );

$rc = post("type=hosttemplate action=edit id=$htid servicechecks=$scid1 exception_${scid1}_add='-H 1 -p 2'");
cmp_ok( $rc,                                                                                                             "==", 0,           "Edited ht with exception added" );
cmp_ok( $dbh->selectrow_array("SELECT COUNT(*) FROM servicecheckhosttemplateexceptions WHERE hosttemplate = $htid"),     "==", 1,           "Only 1 exception" );
cmp_ok( $dbh->selectrow_array("SELECT servicecheck FROM servicecheckhosttemplateexceptions WHERE hosttemplate = $htid"), "==", $scid1,      "And is correct servivecheck" );
cmp_ok( $dbh->selectrow_array("SELECT args FROM servicecheckhosttemplateexceptions WHERE hosttemplate = $htid"),         'eq', '-H 1 -p 2', "Stored exception correctly" );

$rc = post("type=hosttemplate action=edit id=$htid servicechecks=$scid2 exception_${scid2}_add='-H 3 -p 4'");
cmp_ok( $rc,                                                                                                             "==", 0,           "Edited ht with different exception" );
cmp_ok( $dbh->selectrow_array("SELECT COUNT(*) FROM servicecheckhosttemplateexceptions WHERE hosttemplate = $htid"),     "==", 1,           "Still only 1 exception" );
cmp_ok( $dbh->selectrow_array("SELECT servicecheck FROM servicecheckhosttemplateexceptions WHERE hosttemplate = $htid"), "==", $scid2,      "And is correct servivecheck" );
cmp_ok( $dbh->selectrow_array("SELECT args FROM servicecheckhosttemplateexceptions WHERE hosttemplate = $htid"),         'eq', '-H 3 -p 4', "Stored exception correctly" );

$rc = post("type=hosttemplate action=edit id=$htid servicechecks=$scid1 servicechecks=$scid2 exception_${scid2}_add='-m Sting' exception_${scid1}_add='-m Madonna'");
cmp_ok( $rc,                                                                                                                           "==", 0,            "Edited ht with multiple exceptions" );
cmp_ok( $dbh->selectrow_array("SELECT COUNT(*) FROM servicecheckhosttemplateexceptions WHERE hosttemplate = $htid"),                   "==", 2,            "Still only 1 exception" );
cmp_ok( $dbh->selectrow_array("SELECT args FROM servicecheckhosttemplateexceptions WHERE hosttemplate=$htid AND servicecheck=$scid2"), "eq", "-m Sting",   "Stored exception $scid1 correctly" );
cmp_ok( $dbh->selectrow_array("SELECT args FROM servicecheckhosttemplateexceptions WHERE hosttemplate=$htid AND servicecheck=$scid1"), 'eq', '-m Madonna', "Stored exception $scid1 correctly" );

$rc = post("type=hosttemplate action=delete id=$htid");
cmp_ok( $rc,                                                                                                         "==", 0, "Deleted hosttemplate id $htid" );
cmp_ok( $dbh->selectrow_array("SELECT COUNT(*) FROM servicecheckhosttemplateexceptions WHERE hosttemplate = $htid"), "==", 0, "Associated exceptions deleted too" );

