#!/usr/bin/perl
# Test rest login via authtkt

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/lib", "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test qw(opsview);
use Opsview::TestUtils;
use Test::WWW::Mechanize;

use Data::Dump qw(dump);

use JSON;

my $content;
my $expected;
my $ua   = Test::WWW::Mechanize->new( stack_depth => 0 );
$ua->add_header( 'content-type' => "text/x-data-dumper" );

my $url_prefix="http://localhost:3000";

$ua->post( "$url_prefix/rest/login_tkt");
is( $ua->status, 401, "Unauthorized as no username" );
$content = $ua->content;
is( $content, qq%{'message' => 'No username specified for REST login'}%, 'Login error message' ) || diag $content;

$ua->post( "$url_prefix/rest/login_tkt?username=tonnie" );
is( $ua->status, 401, "Unauthorized as no authtkt" );
$content = eval $ua->content;
is_deeply( $content, {'message' => 'Invalid login using authtkt', detail => 'No authtkt found'}, 'Login error message' ) || diag explain $content;

$ua->post( "$url_prefix/rest/login_tkt", Content => qq%{username=>"tonnie"}% );
is( $ua->status, 401, "Unauthorized as no authtkt, username posted as data" );
$content = eval $ua->content;
is_deeply( $content, {'message' => 'Invalid login using authtkt', detail => 'No authtkt found'}, 'Login error message' ) || diag explain $content;

my $newua   = Test::WWW::Mechanize->new( stack_depth => 0 );
log_in( $newua, "admin", "initial", $url_prefix );

$newua->add_header( 'content-type' => "text/x-data-dumper" );

$newua->post( "$url_prefix/rest/login_tkt", Content => qq%{username=>"tonnie"}% );
is( $newua->status, 401, "Unauthorized as username doesn't match ticket" );
$content = eval $newua->content;
is_deeply( $content, {'message' => 'Invalid login using authtkt', 'detail' => "Username 'tonnie' does not match ticket username 'admin'"}, 'Login error message' ) || diag explain $content;


$newua->post( "$url_prefix/rest/login_tkt", Content => qq%{username=>"admin"}% );
is( $newua->status, 200, "Success!" );
$content = $newua->content;
like( $content, qr%^\{'token' => '[0-9a-f]{40}'\}$%, "Got token" );

