#!/usr/bin/perl
# Test rest

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test qw(opsview);
#use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);
use Test::WWW::Mechanize;

use lib "$Bin/lib";
use Opsview::TestUtils;

use Data::Dump qw(dump);

my $content;
#my $ua = Test::WWW::Mechanize::Catalyst->new( stack_depth => 0 );
my $ua = Test::WWW::Mechanize->new( stack_depth => 0 );

my $res_data;

rest_log_in( $ua, "admin", "initial", "http://localhost:3000" );

$ua->put( "http://localhost:3000/rest/config/host", Content => '{ list => [ { id => 1, name => "newone" }, { id => 2, name => "newone" } ] }' );
is( $ua->status, 400, "Caught list error" );
$res_data = eval $ua->content;
is( $res_data->{message}, "Rollback. Error trying to synchronise object: 2", "Got correct error" ) || diag( "Full content: ".$ua->content );
is( $res_data->{objects_updated}, 0, "No objects changed" );

$ua->get( "http://localhost:3000/rest/config/host?s.name=newone" );
is( $ua->status, 200, "Call okay" );
$res_data = eval $ua->content;
is_deeply( $res_data->{list}, [], "No items returned" );

# Sneaky test here to see if other columns can be passed through, eg http_admin_port
$ua->get_ok( "http://localhost:3000/rest/config/host?rows=5&page=1&cols=icon,hosttemplates,ip,name,id,hostgroup,http_admin_port", "Only get 1st page of hosts" );
$res_data = eval $ua->content;
my $expected = {
    'list' => [
        {   'icon'          => { 'name' => 'SYMBOL - Environment Monitor', path => "/images/logos/meter_small.png" },
            'hosttemplates' => [
                {   'ref'  => '/rest/config/hosttemplate/3',
                    'name' => 'Cisco Mgt'
                }
            ],
            'ref'       => '/rest/config/host/7',
            'ip'        => '192.168.10.20',
            'name'      => 'cisco',
            'id'        => '7',
            'hostgroup' => {
                'ref'  => '/rest/config/hostgroup/4',
                'name' => 'Leaf'
            }
        },
        {   'icon'          => { 'name' => 'SYMBOL - *DSL', path => "/images/logos/dsl_small.png" },
            'hosttemplates' => [],
            'ref'           => '/rest/config/host/8',
            'ip'            => '192.168.10.23',
            'name'          => 'cisco1',
            'id'            => '8',
            'hostgroup'     => {
                'ref'  => '/rest/config/hostgroup/4',
                'name' => 'Leaf'
            }
        },
        {   'icon'          => { 'name' => 'SYMBOL - Firewall', path => "/images/logos/firewall_small.png" },
            'hosttemplates' => [],
            'ref'           => '/rest/config/host/9',
            'ip'            => '192.168.10.22',
            'name'          => 'cisco2',
            'id'            => '9',
            'hostgroup'     => {
                'ref'  => '/rest/config/hostgroup/4',
                'name' => 'Leaf'
            }
        },
        {   'icon'          => { 'name' => 'LOGO - NetBSD', path => "/images/logos/netbsd_small.png" },
            'hosttemplates' => [
                {   'ref'  => '/rest/config/hosttemplate/4',
                    'name' => 'Blank'
                }
            ],
            'ref'       => '/rest/config/host/10',
            'ip'        => '192.168.10.22',
            'name'      => 'cisco3',
            'id'        => '10',
            'hostgroup' => {
                'ref'  => '/rest/config/hostgroup/4',
                'name' => 'Leaf'
            }
        },
        {   'icon'          => { 'name' => 'SYMBOL - Network', path => "/images/logos/network_small.png" },
            'hosttemplates' => [],
            'ref'           => '/rest/config/host/11',
            'ip'            => 'not_a_real_host',
            'name'          => 'cisco4',
            'id'            => '11',
            'hostgroup'     => {
                'ref'  => '/rest/config/hostgroup/4',
                'name' => 'Leaf'
            }
        },
    ],
    summary => {
        allrows    => 24,
        page       => 1,
        rows       => 5,
        totalpages => 5,
        totalrows  => 24,
    }
};
is_deeply( $res_data, $expected, "Got expected data for host list" ) || diag( Data::Dump::dump($res_data) );

$ua->get_ok( "http://localhost:3000/rest/config/host?rows=all&s.name=opsview&cols=name", "Check search by name works" );
$res_data = eval $ua->content;
$expected = {
    'list' => [
        {   'name' => 'opsview',
            'ref'  => '/rest/config/host/1',
        },
    ],
    summary => {
        allrows => 24,
        rows    => 1,
    }
};
is_deeply( $res_data, $expected, "Got expected data for host when searching by name" ) || diag( Data::Dump::dump($res_data) );

$ua->get('http://localhost:3000/rest/config/host?rows=all&cols=name&json_filter={"-blah"' );
is( $ua->status, 400, "Got 400 error code for poor JSON");
$res_data = eval $ua->content;
is( $res_data->{message}, "Error when parsing json_filter", "Got right error message" );
like( $res_data->{detail}, qr/\':\' expected, at character offset/, "Got right detail" ) || diag($ua->content);

$ua->get('http://localhost:3000/rest/config/host?rows=all&cols=name&json_filter={"nosuchcolumn":{"-like":"%help%"}}');
is( $ua->status, 400, "Got 400 error code for bad search arguments");
$res_data = eval $ua->content;
is( $res_data->{message}, "Error executing search", "Got right error message" );
like( $res_data->{detail}, qr/Unknown column 'me.nosuchcolumn' in/, "Got right detail" ) || diag($ua->content);

$ua->get_ok( 'http://localhost:3000/rest/config/host?rows=all&cols=name&json_filter={"id":{">":1},"-nest":{"id":{"<":5}}}', "Check search by name works" );
$res_data = eval $ua->content;
$expected = {
    'list' => [
        {   'name' => 'host_locally_monitored',
            'ref'  => '/rest/config/host/3',
        },
        {   'name' => 'monitored_by_slave',
            'ref'  => '/rest/config/host/4',
        },
        {   'name' => 'opsviewdev1',
            'ref'  => '/rest/config/host/2',
        },
    ],
    summary => {
        allrows => 24,
        rows    => 3,
    }
};
is_deeply( $res_data, $expected, "Got right data from complex search" ) || diag(Data::Dump::dump($res_data));


$ua->get_ok( "http://localhost:3000/rest/config/host/7", "Get cisco host info" );
$expected = {
    'object' => {
        'alias'          => '',
        'check_attempts' => '2',
        'check_command'  => {
            'ref'  => '/rest/config/hostcheckcommand/1',
            'name' => 'slow ping'
        },
        'check_interval' => '0',
        'check_period'   => {
            'ref'  => '/rest/config/timeperiod/1',
            'name' => '24x7'
        },
        enable_snmp            => 0,
        flap_detection_enabled => 1,
        hostattributes         => [],
        'hostgroup'            => {
            'ref'  => '/rest/config/hostgroup/4',
            'name' => 'Leaf'
        },
        'hosttemplates' => [
            {   'ref'  => '/rest/config/hosttemplate/3',
                'name' => 'Cisco Mgt'
            }
        ],
        'icon'     => { 'name' => 'SYMBOL - Environment Monitor', path => "/images/logos/meter_small.png" },
        'id'       => '7',
        'ip'       => '192.168.10.20',
        'keywords' => [
            {   'ref'  => '/rest/config/keyword/6',
                'name' => 'alphaearly'
            },
            {   'ref'  => '/rest/config/keyword/2',
                'name' => 'cisco'
            },
            {   'ref'  => '/rest/config/keyword/3',
                'name' => 'cisco_gp1'
            }
        ],
        'monitored_by' => {
            'ref'  => '/rest/config/monitoringserver/1',
            'name' => 'Master Monitoring Server'
        },
        'name'                => 'cisco',
        nmis_node_type        => "router",
        notification_interval => 0,
        notification_options  => "u,d,r",
        notification_period   => {
            name => "24x7",
            ref  => "/rest/config/timeperiod/1",
        },
        'other_addresses' => '',
        'parents'         => [
            {   'ref'  => '/rest/config/host/7',
                'name' => 'cisco'
            }
        ],
        rancid_connection_type => "ssh",
        rancid_password        => undef,
        rancid_username        => undef,
        rancid_vendor          => undef,
        'retry_check_interval' => '1',
        servicechecks          => [
            {   name                => "Another exception",
                event_handler       => undef,
                exception           => undef,
                remove_servicecheck => 0,
                timed_exception     => undef,
                ref                 => "/rest/config/servicecheck/82",
            },
            {   name                => "Coldstart",
                event_handler       => undef,
                exception           => undef,
                remove_servicecheck => 0,
                timed_exception     => undef,
                ref                 => "/rest/config/servicecheck/79"
            },
            {   name                => "Interface",
                event_handler       => undef,
                exception           => undef,
                remove_servicecheck => 0,
                timed_exception     => undef,
                ref                 => "/rest/config/servicecheck/95"
            },
            {   name                => "snmp poll",
                event_handler       => undef,
                exception           => undef,
                remove_servicecheck => 0,
                timed_exception     => undef,
                ref                 => "/rest/config/servicecheck/83"
            },
            {   name                => "Test exceptions",
                event_handler       => undef,
                exception           => undef,
                remove_servicecheck => 0,
                timed_exception     => undef,
                ref                 => "/rest/config/servicecheck/81"
            },
        ],
        snmp_community      => "shouldnotcreate",
        snmp_max_msg_size   => 0,
        snmp_version        => "2c",
        snmp_port           => 161,
        snmpv3_authpassword => "",
        snmpv3_authprotocol => undef,
        snmpv3_privpassword => "",
        snmpv3_privprotocol => undef,
        snmpv3_username     => "",
        uncommitted         => 0,
        use_mrtg            => 0,
        use_nmis            => 0,
        use_rancid          => 0,
        tidy_ifdescr_level  => 0,
    }
};
$res_data = eval $ua->content;
is_deeply( $res_data, $expected, "Got hash data for host as expected" ) || diag( "Full dump: " . Data::Dump::dump( $res_data->{object} ) );

$ua->put( "http://localhost:3000/rest/config/host/7", content => '{ other_addresses => "bahamas", parents => [ { id => 3 } ] }' );
is( $ua->status, 200, "PUT OK" );
$res_data = eval $ua->content;
$expected->{object}->{other_addresses} = "bahamas";
$expected->{object}->{parents} = [ { ref => "/rest/config/host/3", name => "host_locally_monitored" } ];
$expected->{object}->{uncommitted} = 1;
is_deeply( $res_data, $expected, "Got hash data for contact after a PUT to update" );

$expected->{object}->{check_attempts} = 1;
$ua->put( "http://localhost:3000/rest/config/host/7", content => Data::Dump::dump( $expected->{object} ) );
is( $ua->status, 200, "PUT with a lot of data OK" );
$res_data = eval $ua->content;
is_deeply( $res_data, $expected, "Got hash data back with expected data" );

$ua->put( "http://localhost:3000/rest/config/host/7", content => '{ name => "bad%char^s!" }' );
is( $ua->status, 400, "PUT failed" );
$res_data = eval $ua->content;
$expected = { message => "Error trying to synchronise object", };
is( $res_data->{message}, $expected->{message}, "Got right error message" );
is( $res_data->{detail},  "name: Invalid\n",    "Got invalid error message" );

$ua->get_ok( "http://localhost:3000/rest/config/host/17", "Try to get host 17 before delete" );
is( $ua->status, 200, "Host 17 found" );

$ua->request( HTTP::Request::Common::DELETE("http://localhost:3000/rest/config/host/17") );
is( $ua->status, 200, "Delete succeeded" );

note("Try to get host 17 after delete");
$ua->get("http://localhost:3000/rest/config/host/17");
is( $ua->status, 404, "No object - deleted!" );
$ua->content_contains( "No object found with id=17", "Got error message" );

$ua->request( HTTP::Request::Common::DELETE("http://localhost:3000/rest/config/host/1") );
is( $ua->status, 400, "Delete failed due to being opsview master" );
$res_data = eval $ua->content;
is( $res_data->{message}, "Error when deleting object", "Got right error message" );
like( $res_data->{detail}, qr%Cannot delete or update a parent row: a foreign key constraint fails \(`opsview/monitoringservers`, CONSTRAINT `monitoringservers_host_fk`%, "Got right error message" ) || diag( "Output: ". $ua->content );

# Search by id
$ua->get("http://localhost:3000/rest/config/host?s.id=17");
is( $ua->status, 200, "List returns..." );
$res_data = eval $ua->content;
is( $res_data->{summary}->{rows}, 0, "No rows" );

# Need to set Content-Type otherwise WWW::Mechanize will assume post sends form data
$ua->add_header( 'Content-type' => "text/x-data-dumper" );
my $creation = {
    name          => "NewHost",
    ip            => "11.12.13.14",
    hosttemplates => [ { name => "blank" }, { name => "Base Unix" } ],
};
$expected = {
    alias                  => "",
    check_attempts         => 2,
    check_command          => undef,
    check_interval         => 0,
    check_period           => undef,
    enable_snmp            => 0,
    flap_detection_enabled => 1,
    hostattributes         => [],
    hostgroup              => { name => "alphaearly", ref => "/rest/config/hostgroup/9" },
    hosttemplates          => [ { name => "Blank", ref => "/rest/config/hosttemplate/4" }, { name => "Base Unix", ref => "/rest/config/hosttemplate/1" } ],
    icon => { name => "LOGO - Opsview", path => "/images/logos/opsview_small.png" },
    id   => 25,
    ip   => "11.12.13.14",
    ,
    keywords     => [],
    monitored_by => {
        name => "Master Monitoring Server",
        ref  => "/rest/config/monitoringserver/1",
    },
    name                  => "NewHost",
    nmis_node_type        => "router",
    notification_interval => 60,
    notification_options  => undef,
    notification_period   => {
        name => "24x7",
        ref  => "/rest/config/timeperiod/1",
    },
    other_addresses        => "",
    parents                => [],
    rancid_connection_type => "ssh",
    rancid_password        => undef,
    rancid_username        => undef,
    rancid_vendor          => undef,
    retry_check_interval   => 1,
    servicechecks          => [],
    snmp_community         => "",
    snmp_version           => "2c",
    snmp_port              => 161,
    snmpv3_authpassword    => "",
    snmpv3_authprotocol    => undef,
    snmpv3_privpassword    => "",
    snmpv3_privprotocol    => undef,
    snmpv3_username        => "",
    uncommitted            => 1,
    use_mrtg               => 0,
    use_nmis               => 0,
    use_rancid             => 0,
    tidy_ifdescr_level     => 0,
    snmp_max_msg_size      => 0,
};
$ua->post( "http://localhost:3000/rest/config/host", Content => Data::Dump::dump($creation) );
is( $ua->status, 200, "Create succeeded" );
$res_data = eval $ua->content;
is_deeply( $res_data->{object}, $expected, "Expected return of data too" ) || diag( "Full dump: " . Data::Dump::dump( $res_data->{object} ) );

1;
