#!/usr/bin/perl
# Test REST via Opsview::API

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;

use Opsview::Test qw(opsview);
use Test::WWW::Mechanize;

use Data::Dump qw(dump);

use Opsview::API;
use JSON qw(decode_json);

my ($content, $data);

my $ua = Opsview::API->new(
    username => "admin",
    password => "initial",
    api_min_version => "2.0",
    data_format => "json",
    url_prefix => "http://localhost:3000",
);

$ua->login;

$ua->get( "info" );
like( $ua->content, qr/"opsview_edition":"(community|enterprise)"/, "Data in JSON" );

$ua->get( "config/host?s.name=cisco4" );
is( $ua->mech->status, 200 );
$content = $ua->content;
$data = decode_json( $content );
is( $data->{summary}->{rows}, 1,  "Got 1 host" ) || diag( Data::Dump::dump( $data ) );
is( $data->{list}->[0]->{id}, 11,  "With right id" );

$ua->put( "config/host/1", '{"name":"opsview", "alias":"changed by rest api"}' );
is( $ua->mech->status, 200 );
$content = $ua->content;
$data = decode_json( $content );
is( $data->{object}->{id}, 1,  "Got object back" );
is( $data->{object}->{alias}, "changed by rest api",  "changed by rest api" );

eval {
    diag( $ua->post( "config/host", '{"name":"opsview"}' ) );
};
note("Cause a linefeed");
is( $ua->mech->status, 400 );
$content = $ua->content;
$data = decode_json( $content );
is( $data->{message}, "Error trying to create object") || diag(Data::Dump::dump( $data ));
like( $data->{detail}, qr/Duplicate entry/ );

$ua->delete( "config/host/11" );
is( $ua->mech->status, 200, "Delete ok" );
$ua->get( "config/host?s.name=cisco4" );
is( $ua->mech->status, 200 );
$content = $ua->content;
$data = decode_json( $content );
is( $data->{summary}->{rows}, 0 );

