#!/usr/bin/perl
# Test rest for monitoringservers. 3.10 does not allow write access - see 3.11.0 for this functionality

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Test::WWW::Mechanize;

use lib "$Bin/lib";
use Opsview::TestUtils;

use Data::Dump qw(dump);

my $content;
my $ua = Test::WWW::Mechanize->new( stack_depth => 0 );

my $url = "http://localhost:3000";

rest_log_in( $ua, "admin", "initial", $url );

my $res_data;
my $expected = { message => "Method PUT not available" };

$ua->put( "$url/rest/config/monitoringserver/3", content => "{ name => 'bob' }" );
is( $ua->status, 405, "PUT failure" ) || diag($ua->content);
$res_data                       = eval $ua->content;
is_deeply( $res_data, $expected, "No PUT available" ) || diag( Data::Dump::dump($res_data));

$ua->put( "$url/rest/config/monitoringserver", content => "{ name => 'bob' }" );
is( $ua->status, 405, "PUT failure to list" ) || diag($ua->content);
$res_data                       = eval $ua->content;
is_deeply( $res_data, $expected, "No PUT available" ) || diag( Data::Dump::dump($res_data));

$expected->{message} = "Method POST not available";

$ua->post( "$url/rest/config/monitoringserver/3", content => "{ name => 'bob' }" );
is( $ua->status, 405, "POST failure" ) || diag($ua->content);
$res_data                       = eval $ua->content;
is_deeply( $res_data, $expected, "No POST available" ) || diag( Data::Dump::dump($res_data));

$ua->post( "$url/rest/config/monitoringserver", content => "{ name => 'bob' }" );
is( $ua->status, 405, "POST failure to list" ) || diag($ua->content);
$res_data                       = eval $ua->content;
is_deeply( $res_data, $expected, "No POST available" ) || diag( Data::Dump::dump($res_data));

$expected->{message} = "Method DELETE not available";

$ua->request( HTTP::Request::Common::DELETE("$url/rest/config/monitoringserver/3") );
is( $ua->status, 405, "Delete failed correctly" ) || diag($ua->content);
$res_data                       = eval $ua->content;
is_deeply( $res_data, $expected, "No POST available" ) || diag( Data::Dump::dump($res_data));

