#!perl

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";
use lib "/usr/local/nagios/lib";

use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);
use HTTP::Cookies;
use Sys::Hostname;


use Opsview;
use Opsview::Contact;

use Test::More qw(no_plan);
use Test::Deep;

use lib "$Bin/lib";
use Opsview::Web::Test;
use Opsview::TestUtils;

my $admin = Test::WWW::Mechanize::Catalyst->new();
my $ua    = Test::WWW::Mechanize::Catalyst->new();
my $cj    = HTTP::Cookies->new();
$ua->cookie_jar($cj);

# No cookies here - should get login page
log_in( $ua, "somehosts", "somehosts" );

# Login with admin user
log_in( $admin, "admin", "initial" );

# SQL injection via state field
# Needs extra 1 in later versions
$ua->get_ok("/status/host?hostgroupid=1&state=''union%20select%20schema_name,'1','1',1,1,1,1,1,'1',1,1%20from%20information_schema.schemata%23");
$ua->content_contains( "Host Summary &gt; Host Group: Opsview", "Got a host page" );
is( $ua->find_link( text => "information_schema" ), undef, "Did not run SQL injection via link with information_schema as text" );

# HTML tag injection via ack comments
$ua->get_ok("/status/service/acknowledge?filter=unhandled&order=state_desc&order=host&order=service&includeunhandledhosts=1");
$ua->content_contains("Mass Acknowledgements");
$ua->form_name("comment_form");
$ua->field( "comment", "<script>alert('Trick or treat');</script>" );
$ua->tick("service_selection", "monitored_by_slave;VNC");
$ua->current_form->action;
$ua->submit;

$admin->get_ok("/admin/auditlog");
$admin->content_lacks( '<script>', "Should not have any script tags..." );
$admin->content_contains( q{scriptalert('Trick or treat')/script}, "and < > ; are stripped - will also fail if audit logs not added" );

# HTML tag injection via Host comments
$admin->get_ok("/info/host/4/edit");
$admin->content_contains("No comments currently entered");
$admin->form_name("comments");
$admin->field( "content", "Another <script>alert('Halloween surprise!')</script>" );
$admin->current_form->action;
$admin->submit;

$admin->get_ok("/info/host/4");
$admin->content_lacks( '<script>', "Should not have any script tags" );
$admin->content_contains( '&lt;script&gt;', "Script tag escaped" );

$admin->get_ok("/status/service?host=<script>alert(1)</script>");
$admin->content_lacks( '<script>alert(1)</script>', "Script tags escaped" );
