#!/usr/bin/perl
# Needs rc.opsview to be running
# Will restart nagios as part of this test, if not currently running

use Test::More qw(no_plan);

use FindBin qw($Bin);
use lib "$Bin/../lib", "$Bin/../etc";
use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";
use strict;
use Opsview;
use Opsview::Config;
use Test::WWW::Mechanize::Catalyst 'Opsview::Web';
use lib "$Bin/lib";
use Opsview::TestUtils;

# Start up Opsview
$_ = system("/usr/local/nagios/bin/rc.opsview start");
is( $_, 0, "Opsview started up" );

use Sys::Hostname;
my $hostname = Sys::Hostname::hostname;
my $uri      = "http://localhost";

my %login = ( login_username => "admin", login_password => "initial" );

my $mech = Test::WWW::Mechanize::Catalyst->new;

log_in( $mech, 'admin', 'initial' );

# Delete the slave monitoring server - need to delete the monitoring server from the role first
$mech->post_ok( "$uri/admin/role/save/18", { accesses => [ 4, 7, 6, 2 ], hostgroups => [ 7, 6 ], name => "View some, change some - somehosts", }, "POST to remove monitoring server from view some, change some role" );
is( $mech->ct, "text/html" );

$mech->post_ok( "$uri/admin/monitoringserver/delete/2", {}, "POST to delete monitoring server" );
is( $mech->ct, "text/html" );

$mech->post_ok( "$uri/admin/monitoringserver/delete/3", {}, "POST to delete clustered monitoring server" );
is( $mech->ct, "text/html" );

my $starttime = time;

$mech->post_ok( "$uri/admin/reload/do_reload", {%login}, "Reloading" );
$mech->content_contains( " ", "Right content reply" );

note("Sleeping for 45 seconds to allow reload to complete and backup created");
sleep 45;

# Get the backup id
my ( $auditlogid, $reloadid ) = Opsview->db_Main->selectrow_array("SELECT id, reloadid FROM auditlogs WHERE reloadid IS NOT NULL ORDER BY id DESC LIMIT 1");
die "Backup not found" unless $reloadid;
note("For backup, auditlogid=$auditlogid reloadid=$reloadid");

my $backupfile = Opsview::Config->backupfile($reloadid);
my $modtime    = ( stat $backupfile )[9];
cmp_ok( $starttime, "<", $modtime, "Backup file created successfully" );

# Delete host, to prove db restored
$mech->post_ok( "$uri/admin/host/delete/2", {%login}, "POST to delete host id 2" );
$mech->content_contains( 'http-equiv="refresh"', "Contents correct" );

is( Opsview->db_Main->selectrow_array("SELECT COUNT(*) FROM hosts WHERE id=2"), 0, "Host deleted - check restored..." );

# Need to do this first, because will get redirected back here after the backup has restored
$mech->get_ok( "$uri/admin/auditlog", {}, "Get audit logs" );
$mech->content_contains( "Audit Log &gt; List", "Got right page" );

$mech->post_ok( "$uri/admin/backup/restore/$reloadid", {%login}, "Restoring backup ..." );
$mech->content_contains( "Backup restored from", "Backup restored correctly" ) || diag( "Full content: ".$mech->content );

is( Opsview->db_Main->selectrow_array("SELECT COUNT(*) FROM hosts WHERE id=2"), 1, "Host restored!!!" );

# Check that DB has auditlog for the backup. This means that the complete DB was restored (we hope) with the current audit logs retained
is( Opsview->db_Main->selectrow_array("SELECT COUNT(*) FROM auditlogs WHERE id=$auditlogid"), 1, "Audit logs table was not overwritten" );
